﻿CompilerIf Defined(INCLUDE_SMARTERASE, #PB_Constant)=0
#INCLUDE_SMARTERASE=1
;/////////////////////////////////////////////////////////////////////////////////
;***SmartErase***
;*===============
;*
;*©nxSoftWare (www.nxSoftware.com) 2009.
;*======================================
;*  Stephen Rodriguez (srod).
;*
;*  Created with Purebasic 4.3.
;*  Platforms:  Windows.
;*
;*License - DAYLIKE   (Do As You Like with it!)
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;*NOTES.
;*
;*  i)    Enable SmartErase on an ividividual window or parent control in an attempt to reduce
;*        flicker with the child controls when the main window is dynamically resized etc.
;*
;*        Use the command  :  SetSmartErase(hWnd, color, flag)
;*        to enable SmartErase() on the given window/control
;*
;*        where :   hWnd = the window's handle of the window/control in question,
;*        and       color is the background color used to erase the window/control or it can take the following value :
;*                      #SmartErase_USESTANDARDWINDOWCOLOR
;*
;*        and       flag assumes one of the following values :
;*                      #SmartErase_NOERASEATALL
;*                  or  #SmartErase_REMOVESMARTERASE
;*                  or  #SmartErase_MAXERASELEVEL
;*                  or  #SmartErase_ERASELEVEL0   (default)
;*                  or  #SmartErase_MAXERASELEVEL1
;*                  or  #SmartErase_MAXERASELEVEL2
;*                  etc.

;*  ii)   Using SmartErase() may or may not reduce flicker in individual cases. Combine with other techniques as appropriate.
;*
;*  iii)  This utility is not suitable for windows/controls with a lot of child controls!!!
;/////////////////////////////////////////////////////////////////////////////////


EnableExplicit

Declare.i SmartErase_wndProc(hWnd, uMsg, wParam, lParam)


;=================================================================================
;-Constants and structures.
  #SmartErase_PROP$ = "SmartErase_ptr"
  #SmartErase_IGNORECHILD = "SmartErase_IgnoreChild"
  Enumeration
    #SmartErase_USESTANDARDWINDOWCOLOR  = -1
  EndEnumeration
  Enumeration ;Flag.
    #SmartErase_NOERASEATALL            = 1
    #SmartErase_REMOVESMARTERASE
    #SmartErase_MAXERASELEVEL
    #SmartErase_ERASELEVEL0             ;(Default)
    #SmartErase_ERASELEVEL1
    #SmartErase_ERASELEVEL2
    #SmartErase_ERASELEVEL3
    #SmartErase_ERASELEVEL4
    #SmartErase_ERASELEVEL5
    #SmartErase_ERASELEVEL6
  EndEnumeration
  

  Structure _smartErase
    hWnd.i
    oldProc.i
    rgn.i
    flag.i
    rc.RECT
    color.l
  EndStructure
;=================================================================================


;-The main function.

;=================================================================================
;Returns zero if an error.
Procedure.i SetSmartErase(hWnd, color=#SmartErase_USESTANDARDWINDOWCOLOR, flag=#SmartErase_ERASELEVEL0)
  Protected result, *smartErase._smartErase
  If IsWindow_(hWnd) And flag > 0
  ;Check that SmartErase has not already been enabled for this window/control.
    *smartErase = GetProp_(hWnd, #SmartErase_PROP$)
    If *smartErase
      If flag = #SmartErase_REMOVESMARTERASE
        RemoveProp_(hWnd, #SmartErase_PROP$)
        SetWindowLongPtr_(hWnd, #GWL_WNDPROC, *smartErase\oldProc)
        FreeMemory(*smartErase)
        result = #True
      Else
        *smartErase\flag = flag
        If color = #SmartErase_USESTANDARDWINDOWCOLOR
          color = GetSysColor_(#COLOR_3DFACE)
        EndIf
        *smartErase\color = color
        InvalidateRect_(hWnd, 0, 1)
        result = #True
      EndIf
    Else
      *smartErase = AllocateMemory(SizeOf(_smartErase))
      If *smartErase 
        *smartErase\hWnd = hWnd
        If SetProp_(hWnd, #SmartErase_PROP$, *smartErase)
          *smartErase\oldProc = SetWindowLongPtr_(hWnd, #GWL_WNDPROC, @SmartErase_wndProc())
          *smartErase\flag = flag
          If color = #SmartErase_USESTANDARDWINDOWCOLOR
            color = GetSysColor_(#COLOR_3DFACE)
          EndIf
          *smartErase\color = color
          result = #True
        EndIf
      EndIf
    EndIf
  EndIf
  ProcedureReturn result
EndProcedure
;=================================================================================


;-UTILITY FUNCTIONS.

;=================================================================================
;The following enumchilds procedure is used when a more thorough investigation of the window/control is requested.
Procedure.i SmartErase_deepErase(hWnd, *smartErase._smartErase)
  Protected rc.RECT, rgn
  If IsWindowVisible_(hWnd) And GetProp_(hWnd, #SmartErase_IGNORECHILD) = 0
     GetWindowRect_(hWnd, rc)
      ;Only proceed if this rectangle does not wholly contain the original window rect.
        rc\right+1 : rc\bottom+1
        If PtInRect_(rc, *smartErase\rc\top<<32+*smartErase\rc\left) = 0 Or PtInRect_(rc, *smartErase\rc\bottom<<32+*smartErase\rc\right) = 0
          rc\right-1 : rc\bottom-1
          MapWindowPoints_(0, *smartErase\hWnd, rc, 2)
          rgn = CreateRectRgn_(rc\left, rc\top, rc\right, rc\bottom)
          If rgn
            CombineRgn_(*smartErase\rgn, *smartErase\rgn, rgn, #RGN_DIFF)
            DeleteObject_(rgn)
          EndIf
        EndIf
  EndIf
  ProcedureReturn #True
EndProcedure
;=================================================================================


;=================================================================================
;The following enumchilds procedure removes all children from the client region.
Procedure.i SmartErase_enumChilds(hWnd, *smartErase._smartErase)
  Protected rc.RECT, rgn
  If IsWindowVisible_(hWnd) And GetProp_(hWnd, #SmartErase_IGNORECHILD) = 0
   If *smartErase\hWnd = GetParent_(hWnd)
      GetWindowRect_(hWnd, rc)
      MapWindowPoints_(0, *smartErase\hWnd, rc, 2)
      rgn = CreateRectRgn_(rc\left, rc\top, rc\right, rc\bottom)
      If rgn
        CombineRgn_(*smartErase\rgn, *smartErase\rgn, rgn, #RGN_DIFF)
        DeleteObject_(rgn)
      EndIf
    EndIf
  EndIf
  ProcedureReturn #True
EndProcedure
;=================================================================================


;=================================================================================
;The window procedure for all subclassed windows/controls.
Procedure.i SmartErase_wndProc(hWnd, uMsg, wParam, lParam)
  Protected result, *smartErase._smartErase, oldProc, rc.RECT, brush, oldBrush, parenthWnd, i, t1, newhWnd
  *smartErase = GetProp_(hwnd, #SmartErase_PROP$)
  If *smartErase
    oldProc = *smartErase\oldProc
    Select uMsg
      Case #WM_NCDESTROY
        RemoveProp_(hWnd, #SmartErase_PROP$)
        FreeMemory(*smartErase)
        result = CallWindowProc_(oldProc , hWnd, uMsg, wParam, lParam)
      Case #WM_ERASEBKGND
        If wParam And *smartErase\flag <> #SmartErase_NOERASEATALL
          GetClientRect_(hWnd, rc)
          *smartErase\rgn = CreateRectRgn_(rc\left, rc\top, rc\right, rc\bottom) ;This will 
          If *smartErase\rgn
            If *smartErase\flag < #SmartErase_MAXERASELEVEL Or *smartErase\flag = #SmartErase_ERASELEVEL0
              EnumChildWindows_(hWnd, @SmartErase_enumChilds(), *smartErase)
            Else
              newhWnd = hWnd
              If *smartErase\flag = #SmartErase_MAXERASELEVEL
                parenthWnd = GetParent_(newhWnd)
                While parenthWnd
                  newhWnd = parenthWnd
                  parenthWnd = GetParent_(newhWnd)
                Wend
              Else
                t1 = *smartErase\flag - #SmartErase_ERASELEVEL0
                For i = 1 To t1
                  parenthWnd = GetParent_(newhWnd)
                  If parenthWnd
                    newhWnd = parenthWnd
                  Else
                    Break
                  EndIf
                Next
              EndIf
              If newhWnd = hWnd
                EnumChildWindows_(hWnd, @SmartErase_enumChilds(), *smartErase)
              Else
                GetWindowRect_(hWnd, *smartErase\rc)
                EnumChildWindows_(newhWnd, @SmartErase_DeepErase(), *smartErase)
              EndIf
            EndIf
            brush = CreateSolidBrush_(*smartErase\color)
            oldBrush = SelectObject_(wParam, brush)
              FillRgn_(wParam, *smartErase\rgn, brush)
            SelectObject_(wParam, oldBrush)
            DeleteObject_(brush)
            DeleteObject_(*smartErase\rgn)
            *smartErase\rgn = 0
          EndIf
        EndIf
        result = 1
      Default
        result = CallWindowProc_(oldProc , hWnd, uMsg, wParam, lParam)
    EndSelect
  Else
    result = DefWindowProc_(hWnd, uMsg, wParam, lParam)
  EndIf
  ProcedureReturn result
EndProcedure
;=================================================================================

DisableExplicit

CompilerEndIf

; IDE Options = PureBasic 4.30 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 52
; FirstLine = 27
; Folding = -
; EnableUnicode
; EnableThread
; Executable = nxReportU.dll